﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using System.Collections;

namespace Celeste.Mod.RubysEntities
{
    class AltHeightDisplay : Entity
    {
        // Token: 0x170002C4 RID: 708
        // (get) Token: 0x060013F6 RID: 5110 RVA: 0x00052290 File Offset: 0x00050490
        private bool drawText
        {
            get
            {
                return this.altitude >= 0 && this.ease > 0f && !string.IsNullOrEmpty(this.text);
            }
        }

        // Token: 0x060013F7 RID: 5111 RVA: 0x000522B8 File Offset: 0x000504B8
        public AltHeightDisplay(int altitude, int approach_from, float approach_speed, string surrounding_text) : base()
        {
            this.approach_speed = approach_speed;
            this.text = "";
            this.leftText = "";
            this.rightText = "";
            this.easingCamera = true;
            base.Tag = (Tags.HUD | Tags.Persistent);
            this.altitude = altitude;

            if (altitude >= 0 && Dialog.Has(surrounding_text))
            {
                this.surrounding_text = Dialog.Get(surrounding_text, null).Replace("((count))", "{X}");

                this.text = this.surrounding_text;
                this.height = altitude;
                this.approach = (float)approach_from;
                int num = this.text.IndexOf("{X}");
                this.leftText = this.text.Substring(0, num);
                this.leftSize = ActiveFont.Measure(this.leftText).X;
                this.rightText = this.text.Substring(num + 3);
                this.numberSize = ActiveFont.Measure(this.height.ToString()).X;
                this.rightSize = ActiveFont.Measure(this.rightText).X;
                this.size = ActiveFont.Measure(this.leftText + this.height + this.rightText);
            }
            base.Add(new Coroutine(this.Routine(), true));
        }

        // Token: 0x060013F8 RID: 5112 RVA: 0x00052438 File Offset: 0x00050638
        public override void Added(Scene scene)
        {
            base.Added(scene);
            this.spawnedLevel = (scene as Level).Session.Level;
        }

        // Token: 0x060013F9 RID: 5113 RVA: 0x00052457 File Offset: 0x00050657
        private IEnumerator Routine()
        {
            Player player;
            for (;;)
            {
                player = this.Scene.Tracker.GetEntity<Player>();
                if (player != null && (this.Scene as Level).Session.Level != this.spawnedLevel)
                {
                    break;
                }
                yield return null;
            }
            //this.StepAudioProgression();
            //this.easingCamera = false;
            yield return 0.1f;
            //this.Add(new Coroutine(this.CameraUp(), true));
            if (!string.IsNullOrEmpty(this.text) && this.altitude >= 0)
            {
                Audio.Play("event:/game/07_summit/altitude_count");
            }

            while ((this.ease += Engine.DeltaTime / 0.15f) < 1f)
            {
                yield return null;
            }

            while (this.approach < (float)this.height && !player.OnGround(1))
            {
                yield return null;
            }

            this.approach = (float)this.height;
            this.pulse = 1f;
            while ((this.pulse -= Engine.DeltaTime * 4f) > 0f)
            {
                yield return null;
            }
            this.pulse = 0f;
            yield return 1f;
            while ((this.ease -= Engine.DeltaTime / 0.15f) > 0f)
            {
                yield return null;
            }
            this.RemoveSelf();
            yield break;
        }
        
        private IEnumerator CameraUp()
        {
            this.easingCamera = true;
            Level level = this.Scene as Level;
            for (float p = 0f; p < 1f; p += Engine.DeltaTime * 1.5f)
            {
                level.Camera.Y = (float)(level.Bounds.Bottom - 180) + 64f * (1f - Ease.CubeOut(p));
                yield return null;
            }
            yield break;
        }
        
        public override void Update()
        {
            if (this.altitude >= 0 && this.ease > 0f)
            {
                float delta = (float)this.height - this.approach;
                this.approach += delta * approach_speed * Engine.DeltaTime;
                
                if (delta > 0f && delta < 25f)
                {
                    this.approach += 10f * Engine.DeltaTime;
                }
                else if (delta <= 0f)
                {
                    this.approach = (float)this.height;
                }
            }
            Level level = base.Scene as Level;
            if (!this.easingCamera)
            {
                level.Camera.Y = (float)(level.Bounds.Bottom - 180 + 64);
            }
            base.Update();
        }
        
        public override void Render()
        {
            if (this.drawText)
            {
                Vector2 vector = new Vector2(1920f, 1080f) / 2f;
                float scaleFactor = 1.2f + this.pulse * 0.2f;
                Vector2 vector2 = this.size * scaleFactor;
                float num = Ease.SineInOut(this.ease);
                Vector2 vector3 = new Vector2(1f, num);
                Draw.Rect(vector.X - (vector2.X + 64f) * 0.5f * vector3.X, vector.Y - (vector2.Y + 32f) * 0.5f * vector3.Y, (vector2.X + 64f) * vector3.X, (vector2.Y + 32f) * vector3.Y, Color.Black);
                Vector2 vector4 = vector + new Vector2(-vector2.X * 0.5f, 0f);
                Vector2 scale = vector3 * scaleFactor;
                Color color = Color.White * num;
                ActiveFont.Draw(this.leftText, vector4, new Vector2(0f, 0.5f), scale, color);
                ActiveFont.Draw(this.rightText, vector4 + Vector2.UnitX * (this.leftSize + this.numberSize) * scaleFactor, new Vector2(0f, 0.5f), scale, color);
                ActiveFont.Draw(((int)this.approach).ToString(), vector4 + Vector2.UnitX * (this.leftSize + this.numberSize * 0.5f) * scaleFactor, new Vector2(0.5f, 0.5f), scale, color);
            }
        }
        
        public override void Removed(Scene scene)
        {
            base.Removed(scene);
        }
        
        private int altitude;

        private string surrounding_text;
        private string text;
        
        private string leftText;
        
        private string rightText;
        
        private float leftSize;
        
        private float rightSize;
        
        private float numberSize;
        
        private Vector2 size;
        
        private int height;
        
        private float approach;
        
        private float ease;

        private float pulse;
        
        private string spawnedLevel;

        private float approach_speed;
        
        private bool easingCamera;
    }
}
