﻿using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Celeste.Mod.DSidesHelper {
    public class DSidesModule : EverestModule {

        public override Type SettingsType => null;

        public override void Load() {
            Everest.Events.Level.OnLoadEntity += OnLoadEntity;
            On.Celeste.CassetteBlock.Awake += OnCassetteBlockAwake;
        }

        public override void Unload() {
            Everest.Events.Level.OnLoadEntity -= OnLoadEntity;
            On.Celeste.CassetteBlock.Awake -= OnCassetteBlockAwake;
        }

        private bool OnLoadEntity(Level level, LevelData levelData, Vector2 offset, EntityData entityData) {
            if (!entityData.Name.StartsWith("dsides/"))
                return false;
            string name = entityData.Name.Substring(7);

            switch (name) {
                case "invisibleCassetteBlock":
                    CassetteBlock cassetteBlock = new CassetteBlock(entityData, offset) {
                        new InvisibleCassetteBlockComponent()
                    };
                    level.Add(cassetteBlock);

                    // Special cassette block glue code.

                    level.HasCassetteBlocks = true;
                    if (level.CassetteBlockTempo == 1f)
                        level.CassetteBlockTempo = cassetteBlock.Tempo;
                    level.CassetteBlockBeats = Math.Max(cassetteBlock.Index + 1, level.CassetteBlockBeats);
                    // This dies when there still are vanilla cassette blocks after the invisible ones.
                    /*
                    if (!level.Entities.Any(e => e is CassetteBlockManager) &&
                        !level.Entities.GetToAdd().Any(e => e is CassetteBlockManager) &&
                        level.Tracker.GetEntity<CassetteBlockManager>() == null && (level.Session.Area.Mode != AreaMode.Normal || !level.Session.Cassette))
                        level.Add(new CassetteBlockManager());
                    */
                    return true;

                default:
                    return false;
            }
        }

        private void OnCassetteBlockAwake(On.Celeste.CassetteBlock.orig_Awake orig, CassetteBlock self, Scene scene) {
            orig(self, scene);
            if (self.Get<InvisibleCassetteBlockComponent>() == null)
                return;
            scene.Entities.ToAdd.Remove(new DynData<CassetteBlock>(self).Get<Entity>("side"));
            // RemoveAll dies.
            // self.Components.RemoveAll<Image>();
            foreach (Image i in self.Components.GetAll<Image>().ToList())
                self.Components.Remove(i);
        }

    }
}
