﻿// Example usings.
using Celeste.Mod.UI;
using FMOD.Studio;
using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;

namespace Celeste.Mod.RubysEntities
{
    public class RubysEntitiesModule : EverestModule
    {

        // Only one alive module instance can exist at any given time.
        public static RubysEntitiesModule Instance;

        public RubysEntitiesModule()
        {
            Instance = this;
        }

        // If you don't need to store any settings, => null
        public override Type SettingsType => typeof(RubysEntitiesSettings);
        public static RubysEntitiesSettings Settings => (RubysEntitiesSettings)Instance._Settings;

        public static SpriteBank SpriteBank;

        // If you don't need to store any save data, => null
        public override Type SaveDataType => null;

        // Set up any hooks, event handlers and your mod in general here.
        // Load runs before Celeste itself has initialized properly.
        public override void Load()
        {
            Everest.Events.Level.OnLoadEntity += Level_OnLoadEntity;
        }

        // Optional, initialize anything after Celeste has initialized itself properly.
        public override void Initialize()
        {
        }

        // Optional, do anything requiring either the Celeste or mod content here.
        public override void LoadContent(bool firstLoad)
        {
            SpriteBank = new SpriteBank(GFX.Game, "Graphics/RubysEntities/Sprites.xml");
        }

        // Unload the entirety of your mod's content, remove any event listeners and undo all hooks.
        public override void Unload()
        {
            Everest.Events.Level.OnLoadEntity -= Level_OnLoadEntity;
        }

        private bool Level_OnLoadEntity(Level level, LevelData levelData, Vector2 offset, EntityData entityData)
        {
            if (!entityData.Name.StartsWith("rubysentities/"))
                return false;
            string name = entityData.Name.Substring(14);

            switch (name)
            {
                case "heightdisplaytrigger":
                    level.Add(new HeightDisplayTrigger(entityData, offset));
                    return true;
                default:
                    return false;
            }
        }

        private void call_private_method(System.String name, object obj, object[] parameters)
        {
            Tuple<string, object> method_descriptor = new Tuple<string, object>(name, obj);
            MethodInfo method;

            if (!method_lookup.ContainsKey(method_descriptor))   //if method is not yet in the lut
            {
                method = obj.GetType().GetMethod(name, BindingFlags.NonPublic | BindingFlags.Instance);
                method_lookup.Add(method_descriptor, method);
            }

            if (!method_lookup.TryGetValue(method_descriptor, out method))
            {
                throw new Exception("No method \"" + name + "\" of object \"" + obj.GetType().ToString() + "\" was found.");
            }

            method.Invoke(obj, parameters);
        }

        private void set_private_member(System.String name, object obj, object value)
        {
            Tuple<string, object> field_descriptor = new Tuple<string, object>(name, obj);
            FieldInfo field;

            if (!field_lookup.ContainsKey(field_descriptor))   //if field is not yet in the lut
            {
                field = obj.GetType().GetField(name, BindingFlags.NonPublic | BindingFlags.Instance);
                field_lookup.Add(field_descriptor, field);
            }

            if (!field_lookup.TryGetValue(field_descriptor, out field))
            {
                throw new Exception("No field \"" + name + "\" of object \"" + obj.GetType().ToString() + "\" was found.");
            }

            field.SetValue(obj, value);
        }

        private object get_private_member(System.String name, object obj)
        {
            Tuple<string, object> field_descriptor = new Tuple<string, object>(name, obj);
            FieldInfo field;

            if (!field_lookup.ContainsKey(field_descriptor))   //if field is not yet in the lut
            {
                field = obj.GetType().GetField(name, BindingFlags.NonPublic | BindingFlags.Instance);
                field_lookup.Add(field_descriptor, field);
            }

            if (!field_lookup.TryGetValue(field_descriptor, out field))
            {
                throw new Exception("No field \"" + name + "\" of object \"" + obj.GetType().ToString() + "\" was found.");
            }

            return field.GetValue(obj);
        }

        private static Dictionary<Tuple<string, object>, MethodInfo> method_lookup = new Dictionary<Tuple<string, object>, MethodInfo>();
        private static Dictionary<Tuple<string, object>, FieldInfo> field_lookup = new Dictionary<Tuple<string, object>, FieldInfo>();

    }
}